/* Copyright (c) 2012 Massachusetts Institute of Technology
 * 
 * [Also included are functions derived from derfc in SLATEC
 *  (netlib.org/slatec), which "is in the public domain"
 *  and hence may be redistributed under these or any terms.]
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */

/* This file provides generic Octave wrappers for the different Faddeeva::
   functions.  To wrap a specific function, we create a new .cc file with

   #define FADDEEVA_FUNC ...name of function (without Faddeeva:: namespace)...
   #define FADDEEVA_REAL ...1 if real for real z, 0 if not...
   #define FADDEEVA_DOC ... documentation string...
   #include "Faddeeva_oct.cc"
*/

#include "Faddeeva.hh"

#include <octave/oct.h>

#define PASTEx(a,b) a ## b
#define PASTE(a,b) PASTEx(a,b)
#define STRINGx(a) #a
#define STRING(a) STRINGx(a)

DEFUN_DLD(PASTE(Faddeeva_, FADDEEVA_FUNC), args, nargout, FADDEEVA_DOC)
{
  int nargin = args.length();
  octave_value retval;

  if (nargin < 1 || nargin > 2 || nargout > 1)
    print_usage();
  else if (!args(0).is_matrix_type() && !args(0).is_numeric_type())
    error("Faddeeva_" STRING(FADDEEVA_FUNC) ": z argument must be a number, vector, or matrix");
  else if (nargin == 2 && !args(1).is_real_type())
    error("Faddeeva_" STRING(FADDEEVA_FUNC) ": relerr argument must be real");
  else {
    double relerr = nargin < 2 ? 0 : args(1).double_value();
#if FADDEEVA_REAL
    if (args(0).is_real_type()) {
      NDArray z = args(0).array_value();
      NDArray w(z.dims());
      for (octave_idx_type i = 0; i < z.nelem(); ++i) {
	OCTAVE_QUIT; // handle ctrl-c by user
	w.elem(i) = Faddeeva::FADDEEVA_FUNC(z.elem(i));
      }
      retval = w;
    }
    else
#endif
    {
      ComplexNDArray z = args(0).complex_array_value();
      ComplexNDArray w(z.dims());
      for (octave_idx_type i = 0; i < z.nelem(); ++i) {
	OCTAVE_QUIT; // handle ctrl-c by user
	w.elem(i) = Faddeeva::FADDEEVA_FUNC(z.elem(i), relerr);
      }
      retval = w;
    }
  }

  return retval;
}
